from collections import Counter
from email.message import EmailMessage


class EmailInsights:

    def __init__(self) -> None:
        pass

    def _get_email_structure(self, email_text: EmailMessage) -> str | tuple:
        """
        Shows the structure of the email object.

        Parameters
        ----------
            email_text : email.message.EmailMessage
                the email object in consideration.
        """
        if email_text is None:
            raise Exception(f"Input can not be {None}")
        else:
            if isinstance(email_text, str):
                return email_text
            payload = email_text.get_payload()
            if isinstance(payload, list):
                multipart = ", ".join([self._get_email_structure(sub_email) for sub_email in payload])
                return f"multipart({multipart})"
            else:
                return email_text.get_content_type()

    def count_email_structures(self, emails: list | tuple) -> list[tuple[str, int]]:
        """
        Shows the structures of an email object(s) with their corresponding frequencies in descending order.

        Parameters
        ----------
            emails : list or tuple
                the collection of email objects to be considered.
        """
        if isinstance(emails, (list, tuple)):
            structures = Counter()
            for email_ in emails:
                structure = self._get_email_structure(email_)
                structures[structure] += 1
            return structures.most_common()
        else:
            print(f"Input must be type list or tuple not {type(emails)}")



import unittest
from pathlib import Path
from email_utilities import EmailInsights
from email_utilities import EmailConversion


class EmailInsightsTestCase(unittest.TestCase):

    def setUp(self):
        self.filepath = next(Path("Examples/email").iterdir())
        self.dir = list(Path("Examples/email").iterdir()) + list(Path("Examples/html").iterdir())
        self.email_conversion = EmailConversion()
        self.email = self.email_conversion.load_email(self.filepath)
        self.emails = [self.email_conversion.load_email(file_) for file_ in self.dir]
        self.email_insights = EmailInsights()

    def test__get_email_structure(self):
        result = self.email_insights._get_email_structure(self.email)
        self.assertNotEqual(result, None)

    def test_count_email_structures(self):
        result = self.email_insights.count_email_structures(self.emails)
        self.assertGreaterEqual(len(result), 1)



import email.parser
import email.policy
from pathlib import Path
import re
from html import unescape
from typing import Any


class EmailConversion:
    """
      A class to convert email to text.

      ...

      Attributes
      ----------
      filepath : str or Path object
          the location of the email file to be considered.

      Methods
      -------
      load_email(file_location):
          Loads the email in consideration from file_location.
      """

    def __init__(self, filepath: str | Path = None) -> None:
        self.filepath = filepath

    def load_email(self, file_location: str | Path = None) -> Any:
        """
        Constructs all the necessary attributes for the person object.

        Parameters
        ----------
            file_location : str or Path object
                the location of the file to be loaded.
        """
        if self.filepath:
            file_location = self.filepath
        try:
            with open(file_location, "rb") as file:
                return email.parser.BytesParser(policy=email.policy.default).parse(file)
        except FileNotFoundError:
            print("The file can't be found")
        except TypeError:
            print(f"expected str, bytes or os.PathLike object, not {type(self.filepath)}")

    def _convert_html_to_plain_text(self, email_text: Any = None) -> str:
        """
        Converts html-like email to text.

        Parameters
        ----------
            email_text : email.message.Message
                the email to be converted to text.
        """

        if self.filepath and email_text is None:
            email_text = self.load_email()
        if email_text.get_content_type() == "text/html":
            # noinspection PyBroadException
            try:
                email_text = email_text.get_content()
            except:  # in case of encoding issues
                email_text = str(email_text.get_payload())
            email_text = re.sub('<head.*?>.*?</head>', '', email_text, flags=re.MULTILINE | re.DOTALL | re.IGNORECASE)
            email_text = re.sub('<a\s.*?>', ' HYPERLINK ', email_text, flags=re.MULTILINE | re.DOTALL | re.I)
            email_text = re.sub('<.*?>', '', email_text, flags=re.MULTILINE | re.DOTALL)
            email_text = re.sub(r'(\s*\n)+', '\n', email_text, flags=re.MULTILINE | re.DOTALL)
            return unescape(email_text)
        else:
            print(f"The email is a {email_text.get_content_type()} type instead of html")

    def convert_email_to_text(self, email_text: Any = None) -> Any | str:
        """
        Converts email to text.

        Parameters
        ----------
            email_text : email.message.Message
                the email message object to be converted.
        """

        if self.filepath and email_text is None:
            email_text = self.load_email()
        for component in email_text.walk():
            content_type = component.get_content_type()
            if content_type not in ("text/plain", "text/html"):
                continue
            if content_type == "text/plain":
                return component
            else:
                return self._convert_html_to_plain_text(component)



import unittest
from pathlib import Path
import email
from email_utilities import EmailConversion


class EmailConversionTestCase(unittest.TestCase):

    def setUp(self):
        self.filepath = next(Path("Examples/email").iterdir())
        self.html_filepath = next(Path("Examples/html").iterdir())
        self.html_email_conversion = EmailConversion(filepath=self.html_filepath)
        self.email_conversion = EmailConversion(filepath=self.filepath)
        self.email_conversion_no_filepath = EmailConversion()

    def test_load_email(self) -> None:
        result = self.email_conversion.load_email()
        self.assertIsInstance(result, email.message.EmailMessage)

    def test__convert_html_to_plain_text(self) -> None:
        result = self.html_email_conversion._convert_html_to_plain_text()
        self.assertIsInstance(result, str)

    def test_convert_email_to_text(self) -> None:
        result = self.email_conversion.convert_email_to_text()
        self.assertIsInstance(result, email.message.EmailMessage)


if __name__ == '__main__':
    unittest.main()
